@@ -6,7 +6,7 @@ use warnings;
 
 use Carp 'confess';
 
-our $VERSION   = '0.01';
+our $VERSION   = '0.02';
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Class::MOP::Method';
@@ -15,9 +15,14 @@ sub new {
     my $class   = shift;
     my %options = @_;  
         
+    ($options{package_name} && $options{name})
+        || confess "You must supply the package_name and name parameters";     
+        
     my $self = bless {
         # from our superclass
         '&!body'          => undef,
+        '$!package_name'  => $options{package_name},
+        '$!name'          => $options{name},        
         # specific to this subclass
         '$!is_inline'     => ($options{is_inline} || 0),
     } => $class;
